import logging
import re

from django.conf import settings

logger = logging.getLogger('ext_log')


class PwcLoggingMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        method = request.method
        uri = request.get_raw_uri()
        body = request.body
        response = self.get_response(request)

        allow_methods = settings.THX_PWC_LOG_TRACK_METHODS
        if not allow_methods:
            return response
        if method not in allow_methods:
            return response

        urls_to_exclude = settings.THX_PWC_LOG_TRACK_URLS_TO_EXCLUDE
        if urls_to_exclude:
            if re.findall(r"(?=(" + '|'.join(urls_to_exclude) + r"))", uri):
                return response

        user = request.user
        message = "================================================================================================"
        message = f"{message}\nRequest URL: {uri}"
        message = f"{message}\nRequest METHOD: {method}"
        message = f"{message}\nRequest BODY: {body}"
        message = f"{message}\nRequest USER: {user}"
        logger.info(message)

        return response
